"""
:author: Thomas Delaet <thomas@delaet.org>

Port to Domintell
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import os

MINIMUM_MESSAGE_SIZE = 3
MAXIMUM_MESSAGE_SIZE = MINIMUM_MESSAGE_SIZE + 5000

# pylint: disable-msg=C0103
CommandRegistry = {}

ModuleRegistry = {}

def on_app_engine():
    """
    :return: bool
    """
    if 'SERVER_SOFTWARE' in os.environ:
        server_software = os.environ['SERVER_SOFTWARE']
        if server_software.startswith('Google App Engine') or \
                server_software.startswith('Development'):
            return True
        return False
    return False


def register_command(command_name, command_class):
    """
    :return: None
    """
    assert isinstance(command_name, str)
    assert isinstance(command_class, type)
    if command_name not in CommandRegistry:
        CommandRegistry[command_name] = command_class
    else:
        raise Exception('double registration in command registry {}:{}'.format(command_name, command_class))

def register_module(module_name, module_class):
    """
    :return: None
    """
    assert isinstance(module_name, str)
    assert isinstance(module_class, type)
    if module_name not in ModuleRegistry:
        ModuleRegistry[module_name] = module_class
    else:
        raise Exception("Double registration in module registry {}:{}".format(module_name, module_class))

def register_module_class(module_class):
    register_module(module_class._module_code(), module_class)

# pylint: disable-msg=W0401,C0413
from domipy.message import Message
from domipy.command import Command
from domipy.messages import *

from domipy.module import Module
from domipy.modules import *

from domipy.parser import DomintellParser, ParserError
from domipy.controller import Controller
from domipy.connections import WSSConnection, DomintellConnection



#!/usr/bin/python3
"""
Example code to scan Domintell and return list of installed modules.
"""

import asyncio
import time
import logging
import sys
import os, sys
from config import host
import domipy


def _on_message(message):
    print('received message', message)
    print(message)

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
"""
please create a simple credentials.py:

host = {
    'ADDRESS': '192.168.0.1:17481',
    'USERNAME': '<your username>'
    'PASSWORD': '<your password>'
}

"""

#pylint: disable-msg=C0103
logging.info('Configuring controller for {}'.format(host['ADDRESS']))

controller = domipy.Controller(host['ADDRESS'])
controller.subscribe(_on_message)

logging.info('LOGIN')
controller.login(host['USERNAME'], host['PASSWORD'])

time.sleep(10)
logging.info('Starting scan')

controller.scan(None)

logging.info('Starting sleep')

input("Press enter to exit ")

logging.info('Exiting ...')
controller.stop()



"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy


class SetDigitalOutputMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel, value):
        val = 1 if value > 0 else 0
        print("val={}", val)
        domipy.Command.__init__(self, moduleType, serialNumber, channel, "%I", val)

class SetDigitalOutputOnMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel, "%I")

class SetDigitalOutputOffMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel, "%O")

class TogleDigitalOutputMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel, "")

""" Cover support added """
class SetDigitalOutputOpenMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel*2, "%H")

class SetDigitalOutputCloseMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel*2 + 1, "%L")

class SetDigitalOutputStopMessage(domipy.Command):
    def __init__(self, moduleType, serialNumber, channel):
        domipy.Command.__init__(self, moduleType, serialNumber, channel*2, "%O")



"""
DTRP module status
Output card for the control of 1 to 4 trip switches.

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericDOStatusMessage

TRP_COMMAND_CODE = "TRP"

class DTRPStatusMessage(GenericDOStatusMessage):
    """
    DTRP module status
    """
    def __init__(self, address=None):
        GenericDOStatusMessage.__init__(self, 4)
        self.moduleType = TRP_COMMAND_CODE

domipy.register_command(TRP_COMMAND_CODE, DTRPStatusMessage)



"""
AO status (to be inherited)
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
import logging

AO_COMMAND_CODE = "AO"

class GenericAOStatusMessage(domipy.Message):
    """
    Generic Analog output module status
    """

    def __init__(self, outputCount=1, address=None):
        domipy.Message.__init__(self)
        self.moduleType = AO_COMMAND_CODE
        self.outputCount = outputCount
        self.serialNumber = None
        self.dataType = None
        self.outputs = {}
        self.logger = logging.getLogger('domintell')
        for i in range(0, self.outputCount):
            self.outputs[i] = 0

    def populate(self, serialNumber, dataType, dataString):
        """
        :return: None
        """
        self.serialNumber = serialNumber
        self.dataType = dataType

        for output in range(0, self.outputCount):
            try:
                self.outputs[output] = int(dataString[output * 2: (output * 2) + 2].strip(), 16)
            except ValueError:
                self.logger.debug("Unable to populate data string [%s]", dataString)

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        for output in range(0, self.outputCount):
            if output < len(self.outputs):
                json_dict['output{}'.format(output + 1)] = self.outputs[output]
        return json.dumps(json_dict)

    def is_on(self, output):
        if output < self.outputCount:
            if self.outputs[output] > 0:
                return True
        return False
    
    def get_value(self, output):
        if output < self.outputCount:
            return self.outputs[output]
        return 0

    def get_values(self):
        return self.outputs
        


"""
:author: Thomas Delaet <thomas@delaet.org>

Port to domintell
:authot: Zilvinas Biniseviciud <zilvinas@binis.me>
"""
import string
import json
import domipy
from domipy.module_directory import MODULE_DIRECTORY

class Module(object):
    """
    Abstract class for Domintell hardware modules.
    """
    COMMAND_CODE = 'UNK'
    
    #pylint: disable-msg=R0902
    def __init__(self, serial_number, controller):
        """ C'tor
        Args:
            serial_number (str): module serial number
            controller (Controller): Controller
        """

        # module properties
        self._serial_number = serial_number

        # channel (point) properties
        self._chanel_ids = {}
        self._channel_names = {}
        self._channel_paths = {}
        self._values = {}
        for i in range(0, self.number_of_channels()):
            self._values[i] = 0

        self._callbacks = {}

        self._controller = controller
        self._controller.subscribe(self.on_message)
    
    def get_module_type(self):
        """
        Returns the module model type (full code)

        :return: str
        """
        if self._module_code() in MODULE_DIRECTORY.keys(): 
            return MODULE_DIRECTORY[self._module_code()]['type']
        return "UNK"
    
    def get_module_code(self):
        """
        Returns module model short code (3 letters)
        Is used in domintell protocol
        """
        return self._module_code()
    
    def get_serial_number(self):
        return self._serial_number

    def get_module_name(self):
        """
        Returns the module model type

        :return: str
        """
        if self._module_code() in MODULE_DIRECTORY.keys(): 
            return MODULE_DIRECTORY[self._module_code()]['name']
        return "Unknown"
    
    def get_point_id(self, channel):
        if channel < self.number_of_channels():
            return self._chanel_ids[channel]

    def get_name(self, channel):
        """
        Get name for one of the channels

        :return: str
        """
        if channel < self.number_of_channels():
            return self._channel_names[channel]

    def on_message(self, message):
        """
        Process received message
        """
        if isinstance(message, domipy.ModuleInfoMessage):
            # this is a info messages
            # process & store info
            self._on_info_message(message)
        else:
            # this is regular message
            self._on_message(message)

    def _on_message(self, message):
        pass

    def _on_info_message(self, message):
        if message.serialNumber == self.get_serial_number():
            ch = int(message.channel)
            if ch > 0 and ch <= self.number_of_channels():
                self._channel_names[ch] = message.name
                self._channel_paths[ch] = message.path
                self._chanel_ids[ch] = message.point_id

    def number_of_channels(self):
        """
        Retrieve the number of avaiable channels in this module

        :return: int
        """
        return 1
    
    def turn_on(self):
        raise NotImplementedError
    
    def turn_off(self):
        raise NotImplementedError
    
    def is_on(self, channel):
        raise NotImplementedError

    
    def get_status(self):
        message = domipy.ModuleStatusRequest(self.get_module_code(), self.get_serial_number())
        self._controller.send(message)
        
    def get_io_type(self):
        """
        Get Input Output type
        
        Returns:
         ai - analog input
         ao - analog output
         di - digital input
         do - digital output
         t  - temperature controller
         unk - unknown
        """
        if self._module_code() in MODULE_DIRECTORY.keys(): 
            return MODULE_DIRECTORY[self._module_code()]['io']
        return "unk"

    def is_dimmer(self):
        """
        Check if module is a dimmer
            :param self: 
        """   
        if self.get_io_type() == 'ao':
            return True
        return False
    
    def _load(self):
        message = domipy.ModuleStatusRequest(self.get_module_code(), self._serial_number)
        self._controller.send(message)


    @classmethod
    def _module_code(cls):
        return cls.COMMAND_CODE

    def on_status_update(self, channel, callback):
        """
        Callback to execute on status of update of channel
        """
        if not channel in self._callbacks:
            self._callbacks[channel] = [] 
        self._callbacks[channel].append(callback)

    def to_json_basic(self):
        """
        Create JSON structure with generic attributes
        :return: dict
        """
        return {'name': self.__class__.__name__, 
                'module_type': self.get_module_type(), 
                'module_name':self.get_module_name(),
                'module_code': self.get_module_code(), 
                'serial_number': self._serial_number,
                'channel_ids': (self._chanel_ids), 
                'channel_names': (self._channel_names), 
                'channel_paths': (self._channel_paths) }

    def to_json(self):
        # print(self.to_json_basic)
        return json.dumps(self.to_json_basic()) 



# json_dict = self.to_json_basic()
#         for input in range(0, self.inputCount):
#             if input < len(self.inputs):
#                 json_dict['input{}'.format(input)] = self.inputs[input]
#         return json.dumps(json_dict)



"""
Dimmer control module DDIM01
Controlls:
  DISM04
  DISM08
  DMOV01
  VAR - Binary version only

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
import domipy.messages

class GenericDISM0xModule(domipy.Module):
    """
    Abstract DISM  control module     
    """
    COMMAND_CODE = 'UNK'

    def __init__(self, serial_number, controller):
        domipy.Module.__init__(self, serial_number, controller)

    def is_on(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel] > 0:
                return True
        return False
    
    def get_value(self, channel):
        if channel < self.number_of_channels():
            return self._values[channel]
    
    def sim_long_push(self, channel):
        """
        Simulate long push
        """
        # TODO
        pass

    def sim_long_push_end(self, channel):
        """ 
        Simulate end of long push
        """
        # TODO
        pass

    def sim_short_push(self, channel):
        """
        Simulate short push
        """
        # TODO
        pass

    def sim_short_push_end(self, channel):
        """ 
        Simulate end of short push
        """
        # TODO
        pass

    def number_of_channels(self):
        raise NotImplementedError

    def _on_message(self, message):
        raise NotImplementedError

class DISM04Module(GenericDISM0xModule):

    COMMAND_CODE = 'IS4'

    def number_of_channels(self):
        return 4

    def _on_message(self, message):
        if isinstance(message, domipy.DISM4StatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

class DISM08Module(GenericDISM0xModule):

    COMMAND_CODE = 'IS8'

    def number_of_channels(self):
        return 8

    def _on_message(self, message):
        if isinstance(message, domipy.DISM8StatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))
                    

class DMOV01Module(GenericDISM0xModule):

    COMMAND_CODE = 'DET'

    def number_of_channels(self):
        return 1

    def _on_message(self, message):
        if isinstance(message, domipy.DMOVStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))
        

class DVARModule(GenericDISM0xModule):
    """
    4 LED's driver
    """
    COMMAND_CODE = 'VAR'

    def switch_val(self):
        self._controller.send(domipy.SwitchValueMessage(self.COMMAND_CODE, self._serial_number))

    def number_of_channels(self):
        return 1

    def turn_on(self):
        self.switch_val()
    
    def turn_off(self):
        self.switch_val()
    
    def _on_message(self, message):
        if isinstance(message, domipy.VARStatusMessage):
            self._values = message.get_values()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self.get_value(ch))

domipy.register_module_class(DISM04Module)
domipy.register_module_class(DISM08Module)
domipy.register_module_class(DMOV01Module)
domipy.register_module_class(DVARModule)



"""
BIR module status
Output card to control 8 250V/8A two-pole relays.

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericAOStatusMessage

class DDIMStatusMessage(GenericAOStatusMessage):
    COMMAND_CODE = 'DIM'
    """
    DDIM module status
    """
    def __init__(self, address=None):
        GenericAOStatusMessage.__init__(self, 8)
        self.moduleType = DDIMStatusMessage.COMMAND_CODE

domipy.register_command(DDIMStatusMessage.COMMAND_CODE, DDIMStatusMessage)
